use warnings;
use strict;
use Test::More tests => 20;

use POWER::Utils::IO;

my $PORT    = 42538;
my $IP      = '127.0.0.1';

my $listen  = tcp_server($IP, $PORT);
my $client  = tcp_client($IP, $PORT);
my $server;
Accept($listen, sub { $server = shift });

my ($is_done, $is_eof);
my ($server_buf, $client_buf) = (q{}, q{});

# client start disturbing server

$is_eof     = Sysread($server, $server_buf);
ok(!$is_eof, 'no EOF on server yet');
is($server_buf, '', 'server_buf empty');

$client_buf = 'A';
$is_done    = Syswrite($client, $client_buf);
ok($is_done, '1 byte buf written');
is($client_buf, '', 'client_buf empty');

$is_eof     = Sysread($server, $server_buf);
ok(!$is_eof, 'no EOF on server yet');
is($server_buf, 'A', 'server_buf contain A :)');

$client_buf = 'B' x 102399;
$is_done    = Syswrite($client, $client_buf);
ok($is_done, '102399 bytes buf written');
is($client_buf, '', 'client_buf empty');

$is_eof     = Sysread($server, $server_buf);
ok(!$is_eof, 'no EOF on server yet');
is(length($server_buf), 102400);
like($server_buf, qr/\AAB*\Z/);
is($server_buf, 'A'.('B' x 102399), 'server_buf contain ABBBB... %)');

# now it's time for server to write a little...

$server_buf = 'A'.('B' x 1023999);
$is_done    = Syswrite($server, $server_buf);
ok(!$is_done);
my $len = 1024000 - length($server_buf);
ok($len < 1024000, 'junk partially returned to client');
$is_done    = Syswrite($server, $server_buf);
is($len, 1024000 - length($server_buf), q{can't write more until client will read});

$is_eof     = Sysread($client, $client_buf);
ok(!$is_eof, 'no EOF on client yet');
ok(length($client_buf) < 1024000, 'client partially received it junk');
like($client_buf, qr/\AAB*\Z/, 'yeah, it is my junk! :)');
is($len, length($client_buf), 'client received all data server send');

close $server;
$is_eof     = Sysread($client, $client_buf);
ok($is_eof, 'EOF on client now');

